"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const docker_1 = require("../../docker");
const emitter_1 = require("../../stats/emitter");
const getHost = require("../../hosts/db");
const checkout_1 = require("../checkout");
const push_1 = require("./push");
const log_1 = require("./log");
const task_1 = require("./task");
const types_1 = require("../types");
function buildImage(opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const logFile = yield log_1.createLogFile(opts.app, opts.tag);
        const buildId = path.basename(logFile);
        const logState = {
            stage: 1,
            name: 'checkout'
        };
        const logs = {};
        const emit = (events) => __awaiter(this, void 0, void 0, function* () {
            const { stage, name } = logState;
            if (!logs[stage]) {
                logs[stage] = { step: name, logs: [] };
            }
            logs[stage].logs.push(...events);
            events.forEach(ev => emitter_1.build(buildId, ev));
        });
        // We declare these outside of the try block due to needing these in the catch/finally blocks
        const failCommands = [];
        const completeCommands = [];
        const taskOpts = {
            app: opts.app,
            ref: opts.ref,
            sha: opts.sha,
            state: types_1.State.Building
        };
        const setState = (name, state, props) => {
            logState.stage++;
            logState.name = name;
            if (state === taskOpts.state || !state) {
                return;
            }
            taskOpts.state = state;
            return opts.setState(state, props);
        };
        const execCmd = (cmd) => __awaiter(this, void 0, void 0, function* () {
            yield setState(cmd, null);
            const res = yield task_1.executeTask(taskOpts, cmd);
            emitter_1.build(buildId, `> ${cmd}`);
            emit(res.output);
        });
        try {
            const context = yield checkout_1.checkout(opts.app, opts.sha);
            failCommands.push(...task_1.getCommands(context.task, 'fail'));
            completeCommands.push(...task_1.getCommands(context.task, 'complete'));
            for (const cmd of task_1.getCommands(context.task, 'checkout')) {
                yield execCmd(cmd);
            }
            const buildResult = yield startDockerBuild(context.stream, opts.app, opts.tag, emit, opts.hostId);
            taskOpts.imageId = buildResult.imageId;
            taskOpts.imageTag = opts.tag;
            yield setState('successful', types_1.State.Successful, { imageId: buildResult.imageId });
            for (const cmd of task_1.getCommands(context.task, 'success')) {
                yield execCmd(cmd);
            }
        }
        catch (ex) {
            yield setState('failed', types_1.State.Failed);
            log.error(`Failed to build ${opts.app.name}: ${ex.message || ex}`);
            for (const cmd of failCommands) {
                yield execCmd(cmd);
            }
        }
        finally {
            yield setState('completed', taskOpts.state);
            for (const cmd of completeCommands) {
                yield execCmd(cmd);
            }
            // Remove empty log entries
            Object.keys(logs).forEach(id => {
                const obj = logs[Number(id)];
                obj.logs = obj.logs.filter(entry => !!entry);
            });
            const finalLog = Object.keys(logs).map((id, i) => {
                const log = logs[Number(id)];
                return {
                    id: i + 1,
                    step: log.step,
                    logs: log.logs
                };
            });
            yield appendAsync(logFile, JSON.stringify(finalLog, null, 2));
        }
    });
}
exports.buildImage = buildImage;
function startDockerBuild(stream, app, tag, emit, hostId) {
    return __awaiter(this, void 0, void 0, function* () {
        const host = hostId ? yield getHost.getOne(hostId) : yield getAvailableHost();
        const client = docker_1.default(host);
        const options = {
            t: tag,
            forcerm: true,
            nocache: true,
            dockerfile: app.dockerfile || 'Dockerfile'
        };
        return new Promise((resolve, reject) => {
            client.buildImage(stream, options, (err, buildStream) => __awaiter(this, void 0, void 0, function* () {
                if (err) {
                    log.debug(`[${app.name}] ${err}`);
                    reject(err);
                    emit([`Failed to start build: ${err.message || err}`]);
                    return;
                }
                log.debug(`[${app.name}] `);
                try {
                    const responses = yield log_1.handleBuildStream(buildStream, emit);
                    const imageId = getImageId(responses);
                    resolve({ responses, imageId });
                    push_1.default(host, tag);
                }
                catch (err) {
                    reject(err);
                }
            }));
        });
    });
}
function getAvailableHost() {
    return __awaiter(this, void 0, void 0, function* () {
        /**
         * TODO: Iterate over hosts and find first host with available capacity
         */
        const [host] = yield getHost.getAll();
        if (!host) {
            throw new Error(`Unable to build image: No hosts available`);
        }
        return host;
    });
}
function appendAsync(file, content) {
    return new Promise((resolve, reject) => {
        fs.appendFile(file, content, err => {
            if (err) {
                return reject(err);
            }
            resolve();
        });
    });
}
function getImageId(buildResponses) {
    for (const response of buildResponses) {
        if (!response.aux) {
            continue;
        }
        return response.aux.ID;
    }
    return;
}
//# sourceMappingURL=index.js.map