"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const db = require("../../configuration/db");
const docker_1 = require("../../docker");
const emitter_1 = require("../../stats/emitter");
/**
 * Push a tag to the configured registry (if available)
 * @param tag Non-prefixed image tag -- I.e. Is not prefixed with the registry URL
 */
function push(host, imageName) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = docker_1.default(host);
        const config = yield db.getConfig();
        if (!config.dockerRegistry) {
            return;
        }
        // This is the registry configured by default
        // Do not attempt to push to the default registry as it doesn't exist
        const defaultRegistry = '0.0.0.0:5000';
        if (config.dockerRegistry === defaultRegistry) {
            return;
        }
        const image = client.getImage(imageName);
        const registry = config.dockerRegistry;
        const pushInfo = getRepoTag(registry, imageName);
        if (!pushInfo) {
            return;
        }
        const { repo, tag } = pushInfo;
        yield image.tag({
            repo,
            tag
        });
        const imageToPush = client.getImage(`${repo}:${tag}`);
        log.info(`Pushing to registry: ${repo}:${tag}`);
        const authconfig = yield docker_1.getAuthConfig();
        try {
            const stream = yield imageToPush.push({ insecure: true, authconfig });
            stream.on('error', (err) => {
                log.error(`Failed to push to registry: ${err.message || err}`);
            });
            let isSuccessful = true;
            stream.on('data', data => {
                try {
                    const json = JSON.parse(data);
                    if (!json.errorDetail) {
                        return;
                    }
                    isSuccessful = false;
                    const msg = `Failed to push image ${repo}:${tag}: ${json.errorDetail.message}`;
                    log.error(msg);
                    emitter_1.toast('error', msg);
                }
                catch (ex) {
                    const raw = data.toString();
                    if (raw.indexOf('errorDetail') === -1) {
                        return;
                    }
                    isSuccessful = false;
                    const msg = `Failed to push image ${repo}:${tag}: Raw Event: ${raw}`;
                    log.error(msg);
                    emitter_1.toast('error', msg);
                }
            });
            stream.on('end', () => {
                if (isSuccessful) {
                    log.info(`Pushed to registry: ${repo}:${tag}`);
                    emitter_1.toast('success', `Successfully pushed ${repo}:${tag} to registry`);
                }
            });
        }
        catch (ex) {
            log.error(`Failed to push to registry: ${ex.message || ex}`);
        }
    });
}
exports.default = push;
function getRepoTag(registry, originalTag) {
    const split = originalTag.split(':');
    const repo = split.slice(0, -1).join(':');
    const tag = split.slice(-1)[0];
    if (!repo) {
        log.error(`Invalid image supplied: Did not return a valid repository`);
        return;
    }
    if (!tag) {
        log.error(`Invalid image supplied: Did not return a valid tag`);
    }
    return {
        repo: `${registry}/${repo}`,
        tag
    };
}
//# sourceMappingURL=push.js.map