"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const tags_1 = require("../../git/tags");
const db = require("../db");
const types_1 = require("../types");
const queue_1 = require("./queue");
const util_1 = require("./util");
const emitter_1 = require("../../stats/emitter");
class RemoteMonitor {
    constructor(app) {
        this.app = app;
        this.initialised = false;
        this.debug = (message) => log.debug(`[${this.app.name}] ${message}`);
        this.error = (message) => log.error(`[${this.app.name}] ${message}`);
        this.warn = (message) => log.warn(`[${this.app.name}] ${message}`);
        this.info = (message) => log.info(`[${this.app.name}] ${message}`);
        this.initialise = (isNewApplication, retryOffset = 5) => __awaiter(this, void 0, void 0, function* () {
            if (this.initialised) {
                return;
            }
            try {
                log.debug(`[${this.app.name}] Initialising application tracking`);
                yield this.processBranches(isNewApplication);
                this.initialised = true;
                this.debug(`Tracking application`);
                this.poll();
            }
            catch (ex) {
                this.error(`Failed to initialise tracking. Retrying`);
                if (ex.message) {
                    this.debug(`Message: ${ex.message || 'None provided'}`);
                }
                if (ex.stack) {
                    this.debug(ex.stack);
                }
                setTimeout(() => this.initialise(isNewApplication, retryOffset + 5), retryOffset * 1000);
            }
        });
        this.processBranches = (isNewApplication) => __awaiter(this, void 0, void 0, function* () {
            const app = yield db.one(this.app.id);
            if (!app) {
                return;
            }
            // We must update the applicaiton properties otherwise it's possible to build with incorrect info
            this.app = Object.assign(Object.assign({}, this.app), { name: app.name, label: app.label, repository: app.repository, credentialsId: app.credentialsId, autoBuild: app.autoBuild });
            const tracked = yield db.getRemotes(this.app.id);
            const remotes = (yield tags_1.default(app, true));
            const visited = new Set();
            for (const current of remotes) {
                visited.add(current.ref);
                const existing = tracked.find(track => track.remote === current.ref);
                const action = getBranchAction({ existing, current }, { isNewApplication, initialised: this.initialised, autoBuild: !!app.autoBuild });
                yield this.processBranch(current || existing, action);
            }
            // Deal with deleted branches
            for (const existing of tracked) {
                if (visited.has(existing.remote)) {
                    continue;
                }
                const current = remotes.find(remote => remote.ref === existing.remote);
                const action = getBranchAction({ existing, current }, { isNewApplication, initialised: this.initialised, autoBuild: !!app.autoBuild });
                if (action !== 'deleted') {
                    continue;
                }
                yield this.processBranch({ ref: existing.remote, age: new Date(existing.age), seen: new Date(), sha: existing.sha }, action);
            }
        });
        this.processBranch = (branch, action) => __awaiter(this, void 0, void 0, function* () {
            switch (action) {
                case 'deleted':
                    yield db.removeRemote(this.app.id, branch.ref);
                    this.debug(`'${branch.ref}' deleted from origin`);
                    updateBuildStatus(this.app, branch, types_1.State.Deleted);
                    return;
                case 'failed':
                    yield db.updateRemote(this.app.id, branch.ref, { state: types_1.State.Failed });
                    this.debug(`'${branch.ref}' existing 'in progress' build being marked as failed`);
                    updateBuildStatus(this.app, branch, types_1.State.Failed);
                    return;
                case 'done':
                case 'inactive':
                    return;
                case 'new':
                    this.debug(`Tracking new branch '${branch.ref}'`);
                    yield insertNewRemote(this.app, branch);
                    break;
                case 'change':
                    this.debug(`Updated branch '${branch.ref}'`);
                    yield db.updateRemote(this.app.id, branch.ref, { age: branch.age.toISOString() });
                    break;
            }
            if (isBuildable(this.app, branch)) {
                yield queue_1.queue.add(this.app, branch);
            }
        });
        this.poll = () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.processBranches(false);
            }
            finally {
                // TODO: Configurable polling interval per Application
                setTimeout(() => this.poll(), 15000);
            }
        });
    }
}
exports.RemoteMonitor = RemoteMonitor;
function updateBuildStatus(app, branch, state) {
    emitter_1.buildStatus(app.id, {
        applicationId: app.id,
        sha: branch.sha,
        remote: branch.ref,
        state,
        age: branch.age.toISOString(),
        seen: new Date().toISOString()
    });
}
function insertNewRemote(app, remote) {
    return __awaiter(this, void 0, void 0, function* () {
        yield util_1.insertRemote(app, {
            remote: remote.ref,
            sha: remote.sha,
            age: remote.age.toISOString(),
            seen: new Date().toISOString(),
            state: types_1.State.NotDetermined
        });
    });
}
function getBranchAction(state, opts) {
    const existing = state.existing;
    const current = state.current;
    const { autoBuild, isNewApplication, initialised } = opts;
    if (!current && !existing) {
        throw new Error('Invalid branch action call: No existing or current branch provided');
    }
    if (isNewApplication) {
        return 'inactive';
    }
    const isNoLongerRemote = !!existing && !current;
    if (isNoLongerRemote) {
        return 'deleted';
    }
    const isNewRemote = !existing && !!current;
    if (isNewRemote) {
        return autoBuild ? 'new' : 'inactive';
    }
    // Due due the previous XOR checks, both current and existing definitely exist past this point
    // After a restart, we need to ensure 'waiting' builds are re-queued
    // and 'in progress' builds are marked as failed
    if (!initialised) {
        if (existing.state === types_1.State.Waiting) {
            return 'change';
        }
        if (existing.state === types_1.State.Building) {
            return 'failed';
        }
    }
    if (!isActive(new Date(current.age))) {
        return 'inactive';
    }
    const isChange = current.sha !== existing.sha;
    if (isChange) {
        return autoBuild ? 'change' : 'inactive';
    }
    return 'done';
}
function isBuildable(app, remote) {
    if (!app.autoBuild) {
        return false;
    }
    if (remote.type && remote.type !== 'branch') {
        return false;
    }
    if (!remote.age) {
        return false;
    }
    return isActive(remote.age);
}
function isActive(date) {
    const threshold = new Date();
    // TODO: Retrieve limit from config
    const limit = 7;
    threshold.setDate(threshold.getDate() - limit);
    return date.valueOf() >= threshold.valueOf();
}
//# sourceMappingURL=monitor.js.map