"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const db = require("../db");
const build_image_1 = require("../build-image");
const slug_1 = require("./slug");
const emitter_1 = require("../../stats/emitter");
const db_1 = require("../../configuration/db");
class BuildQueue {
    constructor() {
        this.queue = [];
        // TODO: Retrieve from DB configuration
        // This will eventually be derived from "Hosts"
        // Hosts config will determine how many concurrent builds they support
        // "Host.capacity" will become ultimately mean "Host.concurrentBuildLimit"
        // At the moment we just support builds on "localhost"
        this.maxConcurrent = 2;
        this.poll = () => __awaiter(this, void 0, void 0, function* () {
            try {
                const config = yield db_1.getConfig();
                this.maxConcurrent = config.maxConcurrentBuilds;
                const inProgress = this.queue.filter(item => item.state === types_1.State.Building || item.state === types_1.State.Started);
                const waiting = this.queue.filter(item => item.state === types_1.State.Waiting);
                if (inProgress.length >= this.maxConcurrent) {
                    return;
                }
                const nextBuild = waiting[0];
                if (!nextBuild) {
                    return;
                }
                // We must 'fire and forget' this item to allow paralell builds
                this.build(nextBuild);
            }
            finally {
                setTimeout(() => this.poll(), 500);
            }
        });
        this.build = (item) => __awaiter(this, void 0, void 0, function* () {
            const refSlug = slug_1.default(item.ref);
            const buildTag = `${item.app.label}:${refSlug}`;
            // Not awaiting is intentional here
            // If we awaited, we would only perform one build at a time
            yield updateRemote(item, types_1.State.Started);
            build_image_1.buildImage({
                app: item.app,
                ref: item.ref,
                sha: item.sha,
                tag: buildTag,
                setState: (state, props) => updateRemote(item, state, props)
            });
        });
        this.init();
    }
    init() {
        this.poll();
    }
    add(app, remote) {
        return __awaiter(this, void 0, void 0, function* () {
            const existingItem = this.queue.find(item => item.app.id === app.id && item.ref === remote.ref);
            if (!existingItem) {
                log.debug(`[${app.name}] Add job to build queue '${remote.ref}'`);
                const item = Object.assign(Object.assign({}, remote), { app, state: types_1.State.Waiting });
                this.queue.push(item);
                yield updateRemote(item, types_1.State.Waiting);
                return;
            }
            if (existingItem.sha === remote.sha) {
                // If we are already aware of the SHA and it is 'dealt with', do nothing
                switch (existingItem.state) {
                    case types_1.State.Building:
                    case types_1.State.Waiting:
                        return;
                }
                // Fall through for 'non-dealt with' remotes and re-build them
            }
            // The SHA for the remote may have updated while we were waiting to start
            // Do not build the original SHA and leave its position in the queue
            existingItem.sha = remote.sha;
            log.debug(`[${app.name}] Updated waiting job '${remote.ref}'`);
            yield updateRemote(existingItem, types_1.State.Waiting);
        });
    }
}
exports.queue = new BuildQueue();
function updateRemote(item, state, props = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        emit(item, state, props.imageId);
        item.state = state;
        yield db.updateRemote(item.app.id, item.ref, Object.assign({ state, sha: item.sha, age: item.age.toISOString() }, props));
    });
}
function emit(item, state, imageId) {
    emitter_1.buildStatus(item.app.id, {
        applicationId: item.app.id,
        sha: item.sha,
        state,
        imageId,
        remote: item.ref,
        age: item.age.toISOString(),
        seen: item.seen ? item.seen.toISOString() : undefined
    });
}
//# sourceMappingURL=queue.js.map