"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const childProcess = require("child_process");
const fs_1 = require("fs");
const getCreds = require("../credentials/db");
/**
 * Execute abritrary Git commands in an Application's git repository
 */
function execCommand(application, workingDirectory, command, args) {
    return __awaiter(this, void 0, void 0, function* () {
        if (args && command.split(' ').length > 1) {
            throw new Error('[GITCMD] Cannot provide arguments and command with parameters');
        }
        const creds = application.credentialsId
            ? yield getCreds.one(application.credentialsId)
            : undefined;
        // If the repository is not private, we do not need to use a SSH private key
        if (!creds) {
            const result = yield spawnAsync(command, { cwd: workingDirectory }, undefined, args);
            return result;
        }
        // A script needs to be created and passed to the Git command with instructions on how to consume SSH
        // This is a necessary evil for passing a SSH private key programmatically
        if (isPrivateKey(creds.key)) {
            const filenames = yield createFiles(creds.key);
            const GIT_SSH = filenames.script;
            try {
                const result = yield spawnAsync(command, { detached: true, cwd: workingDirectory, env: { GIT_SSH } }, creds, args);
                yield teardown(filenames);
                return result;
            }
            catch (ex) {
                yield teardown(filenames);
                throw ex;
            }
        }
        const infixedRepo = infixCredentials(application, creds);
        const amendedCommand = command.replace(application.repository, infixedRepo);
        const result = yield spawnAsync(amendedCommand, { cwd: workingDirectory }, creds, args);
        return result;
    });
}
exports.default = execCommand;
function infixCredentials(app, creds) {
    const { username, key } = creds;
    const repository = app.repository;
    const hasUsername = repository.split('@').length > 1;
    // Assumes format: https://username@github.com/org/repo
    if (hasUsername) {
        return repository.split('@').join(`:${key}@`);
    }
    // Assumes formats:
    // - http://private-domain.com/org/repo
    // - https://github.com/org/repo
    return repository
        .replace('http://', `http://${username}:${key}@`)
        .replace('https://', `https://${username}:${key}@`);
}
function isPrivateKey(key) {
    return key.indexOf('-----BEGIN RSA PRIVATE KEY-----') === 0;
}
function createFiles(privateKey) {
    return __awaiter(this, void 0, void 0, function* () {
        // We need to create a file that is guaranteed not to clash with any existing files
        const suffix = randomWord(20);
        const scriptFilename = `/tmp/${suffix}_wrapper.sh`;
        const keyFilename = `/tmp/${suffix}_key.key`;
        const script = `#!/bin/sh\nexec ssh -i ${keyFilename} -o StrictHostKeyChecking=no "$@"`;
        const filenames = { script: scriptFilename, key: keyFilename };
        // Create the script and key file
        yield writeFileAsync(scriptFilename, script);
        yield writeFileAsync(keyFilename, privateKey);
        // Set the correct permissions (script is executable, keyfile is write/read only by owner)
        yield chmodAsync(scriptFilename, '0755');
        yield chmodAsync(keyFilename, '0600');
        return filenames;
    });
}
function teardown(filenames) {
    return __awaiter(this, void 0, void 0, function* () {
        yield unlinkAsync(filenames.key);
        yield unlinkAsync(filenames.script);
        return true;
    });
}
function randomWord(length) {
    // Surprisingly, this very rarely creates an actual word
    let word = '';
    for (let i = 0; i < length; ++i) {
        const code = Math.floor(Math.random() * 26) + 97;
        word += String.fromCharCode(code);
    }
    return word;
}
/**
 * Wrapping Node filesystem functions in promises
 */
function writeFileAsync(filename, data) {
    const promise = new Promise((resolve, reject) => {
        fs_1.writeFile(filename, data, error => {
            if (error) {
                return reject(error);
            }
            resolve(true);
        });
    });
    return promise;
}
function chmodAsync(filename, mode) {
    const promise = new Promise((resolve, reject) => {
        fs_1.chmod(filename, mode, error => {
            if (error) {
                return reject(error);
            }
            resolve(true);
        });
    });
    return promise;
}
function unlinkAsync(filename) {
    const promise = new Promise((resolve, reject) => {
        fs_1.unlink(filename, error => {
            if (error) {
                return reject(error);
            }
            resolve(true);
        });
    });
    return promise;
}
function spawnAsync(command, options, creds, args) {
    const promise = new Promise((resolve, reject) => {
        let buffer = '';
        const buf = (msg) => {
            if (creds) {
                buffer += (msg || '').toString().replace(creds.key, '**********');
                return;
            }
            buffer += (msg || '').toString();
        };
        const split = command.split(' ');
        const baseCommand = split[0];
        const commandArgs = args || split.slice(1);
        const proc = childProcess.spawn(baseCommand, commandArgs, options);
        proc.on('message', msg => buf(msg));
        proc.on('data', (msg) => buf(msg));
        proc.on('close', code => {
            if (code !== 0) {
                return reject(`Process exited with status ${code}\n${buffer}`);
            }
            resolve(buffer);
        });
        proc.on('error', err => buf(err));
        proc.stdout.on('data', data => buf(data));
        proc.stderr.on('data', data => buf(data));
    });
    return promise;
}
//# sourceMappingURL=cmd.js.map