"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const getHosts = require("../hosts/db");
const data_1 = require("../../data");
const docker_1 = require("../docker");
const emitter = require("./emitter");
const analysis = require("analysis");
const db_1 = require("../configuration/db");
const containerStats = {};
function monitorAll() {
    return __awaiter(this, void 0, void 0, function* () {
        const hosts = yield getHosts.getAll();
        for (const host of hosts) {
            const client = docker_1.default(host);
            const containers = yield client.listContainers();
            for (const container of containers) {
                watchContainer(host, container.Id);
            }
        }
    });
}
exports.default = monitorAll;
function watchContainer(host, containerId) {
    if (containerStats[containerId] !== undefined) {
        return;
    }
    const client = docker_1.default(host);
    containerStats[containerId] = {
        json: '',
        cpu: [],
        memory: [],
        hostId: host.id,
        containerId
    };
    client.getContainer(containerId).stats((_, stream) => {
        log.info(`[${containerId.slice(0, 10)}] Monitoring container`);
        stream.on('data', (data) => parseData(containerId, data));
        stream.on('end', () => {
            log.info(`[${containerId.slice(0, 10)}] Stats stream ended`);
            delete containerStats[containerId];
        });
        stream.on('error', err => log.info(`[${containerId.slice(0, 10)}] Stats stream errored: ${err}`));
    });
}
exports.watchContainer = watchContainer;
function parseData(containerId, buffer) {
    const stats = containerStats[containerId];
    const rawJson = stats.json + buffer.toString();
    try {
        const json = JSON.parse(rawJson);
        const memory = getMemory(json);
        const cpu = getCPU(json);
        const io = getDataTransfer(json);
        json.memory = isNaN(memory) ? '...' : memory + '%';
        json.cpu = isNaN(cpu) ? '...' : cpu + '%';
        json.tx = io.tx;
        json.rx = io.rx;
        emitter.containerStats(containerId, json);
        stats.cpu.push(cpu);
        stats.memory.push(memory);
        stats.json = '';
        persist(stats);
    }
    catch (_) {
        stats.json = rawJson;
    }
}
function getCPU(event) {
    const postCpuStats = event.cpu_stats;
    const preCpuStats = event.precpu_stats;
    const x = preCpuStats.cpu_usage.total_usage - postCpuStats.cpu_usage.total_usage;
    const y = preCpuStats.system_cpu_usage - postCpuStats.system_cpu_usage;
    const cpuPercent = analysis.common.round(x / (x + y) * 100, 2);
    return isNaN(cpuPercent) ? 0 : cpuPercent;
}
function getDataTransfer(event) {
    const networks = event.networks || {};
    const eth0 = networks['eth0'];
    if (eth0) {
        return {
            rx: analysis.common.round(eth0.rx_bytes / 1024 / 1024, 2) + 'MB',
            tx: analysis.common.round(eth0.tx_bytes / 1024 / 1024, 2) + 'MB'
        };
    }
    return {
        rx: '...',
        tx: '...'
    };
}
function getMemory(event) {
    const memStats = event.memory_stats;
    const memory = memStats.usage / 1024 / 1024 / (memStats.limit / 1024 / 1024) * 100;
    const memPercent = analysis.common.round(memory, 2);
    return isNaN(memPercent) ? 0 : memPercent;
}
function persist(stats) {
    return __awaiter(this, void 0, void 0, function* () {
        const config = yield db_1.getConfig();
        const binSize = Number(config.statsBinSize);
        // Truncate records older than the retention limit
        const now = new Date();
        const boundary = now.setDate(now.getDate() - config.statsRetentionDays).valueOf();
        yield data_1.heartbeats()
            .delete()
            .where('timestamp', '<', boundary);
        // CPU and Memory will be same length in container stats
        // Use either
        if (stats.cpu.length < binSize) {
            return;
        }
        const cpu = analysis.descriptive.box(stats.cpu);
        const memory = analysis.descriptive.box(stats.memory);
        yield data_1.heartbeats().insert({
            hostId: stats.hostId,
            containerId: stats.containerId,
            cpu: JSON.stringify(cpu),
            memory: JSON.stringify(memory),
            timestamp: Date.now()
        });
        // Reset stats bin
        stats.cpu = [];
        stats.memory = [];
    });
}
//# sourceMappingURL=containers.js.map